@@ -25,73 +25,26 @@
 
 using System;
 using System.Collections.Generic;
-using System.Text;
 using System.IO;
-using System.Xml;
 using Newtonsoft.Json.Utilities;
-using Newtonsoft.Json.Linq;
 using System.Globalization;
+#if NETFX_CORE
+using IConvertible = Newtonsoft.Json.Utilities.Convertible;
+#endif
+#if NET20
+using Newtonsoft.Json.Utilities.LinqBridge;
+#else
+using System.Linq;
+#endif
 
 namespace Newtonsoft.Json
 {
-  /// <summary>
-  /// Specifies the state of the <see cref="JsonWriter"/>.
-  /// </summary>
-  public enum WriteState
-  {
-    /// <summary>
-    /// An exception has been thrown, which has left the <see cref="JsonWriter"/> in an invalid state.
-    /// You may call the <see cref="JsonWriter.Close"/> method to put the <see cref="JsonWriter"/> in the <c>Closed</c> state.
-    /// Any other <see cref="JsonWriter"/> method calls results in an <see cref="InvalidOperationException"/> being thrown. 
-    /// </summary>
-    Error,
-    /// <summary>
-    /// The <see cref="JsonWriter.Close"/> method has been called. 
-    /// </summary>
-    Closed,
-    /// <summary>
-    /// An object is being written. 
-    /// </summary>
-    Object,
-    /// <summary>
-    /// A array is being written.
-    /// </summary>
-    Array,
-    /// <summary>
-    /// A constructor is being written.
-    /// </summary>
-    Constructor,
-    /// <summary>
-    /// A property is being written.
-    /// </summary>
-    Property,
-    /// <summary>
-    /// A write method has not been called.
-    /// </summary>
-    Start
-  }
-
-  /// <summary>
-  /// Specifies formatting options for the <see cref="JsonTextWriter"/>.
-  /// </summary>
-  public enum Formatting
-  {
-    /// <summary>
-    /// No special formatting is applied. This is the default.
-    /// </summary>
-    None,
-    /// <summary>
-    /// Causes child objects to be indented according to the <see cref="JsonTextWriter.Indentation"/> and <see cref="JsonTextWriter.IndentChar"/> settings.
-    /// </summary>
-    Indented
-  }
-
   /// <summary>
   /// Represents a writer that provides a fast, non-cached, forward-only way of generating Json data.
   /// </summary>
   public abstract class JsonWriter : IDisposable
   {
-    private enum State
+    internal enum State
     {
       Start,
       Property,
@@ -107,34 +60,104 @@ namespace Newtonsoft.Json
     }
 
     // array that gives a new state based on the current state an the token being written
-    private static readonly State[][] stateArray = new[] {
-//                      Start                   PropertyName            ObjectStart         Object            ArrayStart              Array                   ConstructorStart        Constructor             Closed          Error
+    private static readonly State[][] StateArray;
+
+    internal static readonly State[][] StateArrayTempate = new[] {
+//                                      Start                   PropertyName            ObjectStart         Object            ArrayStart              Array                   ConstructorStart        Constructor             Closed          Error
 //                        
-/* None             */new[]{ State.Error,            State.Error,            State.Error,        State.Error,      State.Error,            State.Error,            State.Error,            State.Error,            State.Error,    State.Error },
-/* StartObject      */new[]{ State.ObjectStart,      State.ObjectStart,      State.Error,        State.Error,      State.ObjectStart,      State.ObjectStart,      State.ObjectStart,      State.ObjectStart,      State.Error,    State.Error },
-/* StartArray       */new[]{ State.ArrayStart,       State.ArrayStart,       State.Error,        State.Error,      State.ArrayStart,       State.ArrayStart,       State.ArrayStart,       State.ArrayStart,       State.Error,    State.Error },
-/* StartConstructor */new[]{ State.ConstructorStart, State.ConstructorStart, State.Error,        State.Error,      State.ConstructorStart, State.ConstructorStart, State.ConstructorStart, State.ConstructorStart, State.Error,    State.Error },
-/* StartProperty    */new[]{ State.Property,         State.Error,            State.Property,     State.Property,   State.Error,            State.Error,            State.Error,            State.Error,            State.Error,    State.Error },
-/* Comment          */new[]{ State.Start,            State.Property,         State.ObjectStart,  State.Object,     State.ArrayStart,       State.Array,            State.Constructor,      State.Constructor,      State.Error,    State.Error },
-/* Raw              */new[]{ State.Start,            State.Property,         State.ObjectStart,  State.Object,     State.ArrayStart,       State.Array,            State.Constructor,      State.Constructor,      State.Error,    State.Error },
-/* Value            */new[]{ State.Start,            State.Object,           State.Error,        State.Error,      State.Array,            State.Array,            State.Constructor,      State.Constructor,      State.Error,    State.Error },
+/* None                        */new[]{ State.Error,            State.Error,            State.Error,        State.Error,      State.Error,            State.Error,            State.Error,            State.Error,            State.Error,    State.Error },
+/* StartObject                 */new[]{ State.ObjectStart,      State.ObjectStart,      State.Error,        State.Error,      State.ObjectStart,      State.ObjectStart,      State.ObjectStart,      State.ObjectStart,      State.Error,    State.Error },
+/* StartArray                  */new[]{ State.ArrayStart,       State.ArrayStart,       State.Error,        State.Error,      State.ArrayStart,       State.ArrayStart,       State.ArrayStart,       State.ArrayStart,       State.Error,    State.Error },
+/* StartConstructor            */new[]{ State.ConstructorStart, State.ConstructorStart, State.Error,        State.Error,      State.ConstructorStart, State.ConstructorStart, State.ConstructorStart, State.ConstructorStart, State.Error,    State.Error },
+/* StartProperty               */new[]{ State.Property,         State.Error,            State.Property,     State.Property,   State.Error,            State.Error,            State.Error,            State.Error,            State.Error,    State.Error },
+/* Comment                     */new[]{ State.Start,            State.Property,         State.ObjectStart,  State.Object,     State.ArrayStart,       State.Array,            State.Constructor,      State.Constructor,      State.Error,    State.Error },
+/* Raw                         */new[]{ State.Start,            State.Property,         State.ObjectStart,  State.Object,     State.ArrayStart,       State.Array,            State.Constructor,      State.Constructor,      State.Error,    State.Error },
+/* Value (this will be copied) */new[]{ State.Start,            State.Object,           State.Error,        State.Error,      State.Array,            State.Array,            State.Constructor,      State.Constructor,      State.Error,    State.Error }
 		};
 
-    private int _top;
+    internal static State[][] BuildStateArray()
+    {
+      var allStates = StateArrayTempate.ToList();
+      var errorStates = StateArrayTempate[0];
+      var valueStates = StateArrayTempate[7];
 
-    private readonly List<JTokenType> _stack;
+      foreach (JsonToken valueToken in EnumUtils.GetValues(typeof(JsonToken)))
+      {
+        if (allStates.Count <= (int)valueToken)
+        {
+          switch (valueToken)
+          {
+            case JsonToken.Integer:
+            case JsonToken.Float:
+            case JsonToken.String:
+            case JsonToken.Boolean:
+            case JsonToken.Null:
+            case JsonToken.Undefined:
+            case JsonToken.Date:
+            case JsonToken.Bytes:
+              allStates.Add(valueStates);
+              break;
+            default:
+              allStates.Add(errorStates);
+              break;
+          }
+        }
+      }
+
+      return allStates.ToArray();
+    }
+
+    static JsonWriter()
+    {
+      StateArray = BuildStateArray();
+    }
+
+    private readonly List<JsonPosition> _stack;
+    private JsonPosition _currentPosition;
     private State _currentState;
     private Formatting _formatting;
 
+    /// <summary>
+    /// Gets or sets a value indicating whether the underlying stream or
+    /// <see cref="TextReader"/> should be closed when the writer is closed.
+    /// </summary>
+    /// <value>
+    /// true to close the underlying stream or <see cref="TextReader"/> when
+    /// the writer is closed; otherwise false. The default is true.
+    /// </value>
+    public bool CloseOutput { get; set; }
+
     /// <summary>
     /// Gets the top.
     /// </summary>
     /// <value>The top.</value>
     protected internal int Top
     {
-      get { return _top; }
+      get
+      {
+        int depth = _stack.Count;
+        if (Peek() != JsonContainerType.None)
+          depth++;
+
+        return depth;
+      }
     }
 
+    internal string ContainerPath
+    {
+      get
+      {
+        if (_currentPosition.Type == JsonContainerType.None)
+          return string.Empty;
+
+        IEnumerable<JsonPosition> positions = (_currentPosition.InsideContainer())
+          ? _stack
+          : _stack.Concat(new[] { _currentPosition });
+
+        return JsonPosition.BuildPath(positions);
+      }
+    }
+    
     /// <summary>
     /// Gets the state of the writer.
     /// </summary>
@@ -162,13 +185,30 @@ namespace Newtonsoft.Json
           case State.Start:
             return WriteState.Start;
           default:
-            throw new JsonWriterException("Invalid state: " + _currentState);
+            throw JsonWriterException.Create(this, "Invalid state: " + _currentState, null);
         }
       }
     }
 
     /// <summary>
-    /// Indicates how the output is formatted.
+    /// Gets the path of the writer. 
+    /// </summary>
+    public string Path
+    {
+      get
+      {
+        if (_currentPosition.Type == JsonContainerType.None)
+          return string.Empty;
+
+        return JsonPosition.BuildPath(_stack.Concat(new[] { _currentPosition }));
+      }
+    }
+
+    private DateFormatHandling _dateFormatHandling;
+    private DateTimeZoneHandling _dateTimeZoneHandling;
+
+    /// <summary>
+    /// Indicates how JSON text output is formatted.
     /// </summary>
     public Formatting Formatting
     {
@@ -176,37 +216,89 @@ namespace Newtonsoft.Json
       set { _formatting = value; }
     }
 
+    /// <summary>
+    /// Get or set how dates are written to JSON text.
+    /// </summary>
+    public DateFormatHandling DateFormatHandling
+    {
+      get { return _dateFormatHandling; }
+      set { _dateFormatHandling = value; }
+    }
+
+    /// <summary>
+    /// Get or set how <see cref="DateTime"/> time zones are handling when writing JSON.
+    /// </summary>
+    public DateTimeZoneHandling DateTimeZoneHandling
+    {
+      get { return _dateTimeZoneHandling; }
+      set { _dateTimeZoneHandling = value; }
+    }
+
     /// <summary>
     /// Creates an instance of the <c>JsonWriter</c> class. 
     /// </summary>
-    public JsonWriter()
+    protected JsonWriter()
     {
-      _stack = new List<JTokenType>(8);
-      _stack.Add(JTokenType.None);
+      _stack = new List<JsonPosition>(4);
       _currentState = State.Start;
       _formatting = Formatting.None;
+      _dateTimeZoneHandling = DateTimeZoneHandling.RoundtripKind;
+
+      CloseOutput = true;
     }
 
-    private void Push(JTokenType value)
+    private void UpdateScopeWithFinishedValue()
     {
-      _top++;
-      if (_stack.Count <= _top)
-        _stack.Add(value);
+      if (_currentPosition.Type == JsonContainerType.Array
+        || _currentPosition.Type == JsonContainerType.Constructor)
+      {
+        if (_currentPosition.Position == null)
+          _currentPosition.Position = 0;
+        else
+          _currentPosition.Position++;
+      }
+    }
+
+    private void Push(JsonContainerType value)
+    {
+      UpdateScopeWithFinishedValue();
+
+      if (_currentPosition.Type == JsonContainerType.None)
+      {
+        _currentPosition.Type = value;
+      }
       else
-        _stack[_top] = value;
+      {
+        _stack.Add(_currentPosition);
+        var state = new JsonPosition
+        {
+          Type = value
+        };
+        _currentPosition = state;
+      }
     }
 
-    private JTokenType Pop()
+    private JsonContainerType Pop()
     {
-      JTokenType value = Peek();
-      _top--;
+      JsonPosition oldPosition;
+      if (_stack.Count > 0)
+      {
+        oldPosition = _currentPosition;
+        _currentPosition = _stack[_stack.Count - 1];
+        _stack.RemoveAt(_stack.Count - 1);
+      }
+      else
+      {
+        oldPosition = _currentPosition;
+        _currentPosition = new JsonPosition();
+      }
 
-      return value;
+      return oldPosition.Type;
     }
 
-    private JTokenType Peek()
+    private JsonContainerType Peek()
     {
-      return _stack[_top];
+      return _currentPosition.Type;
     }
 
     /// <summary>
@@ -228,13 +320,13 @@ namespace Newtonsoft.Json
     public virtual void WriteStartObject()
     {
       AutoComplete(JsonToken.StartObject);
-      Push(JTokenType.Object);
+      Push(JsonContainerType.Object);
     }
 
     /// <summary>
     /// Writes the end of a Json object.
     /// </summary>
-    public void WriteEndObject()
+    public virtual void WriteEndObject()
     {
       AutoCompleteClose(JsonToken.EndObject);
     }
@@ -245,13 +337,13 @@ namespace Newtonsoft.Json
     public virtual void WriteStartArray()
     {
       AutoComplete(JsonToken.StartArray);
-      Push(JTokenType.Array);
+      Push(JsonContainerType.Array);
     }
 
     /// <summary>
     /// Writes the end of an array.
     /// </summary>
-    public void WriteEndArray()
+    public virtual void WriteEndArray()
     {
       AutoCompleteClose(JsonToken.EndArray);
     }
@@ -263,13 +355,13 @@ namespace Newtonsoft.Json
     public virtual void WriteStartConstructor(string name)
     {
       AutoComplete(JsonToken.StartConstructor);
-      Push(JTokenType.Constructor);
+      Push(JsonContainerType.Constructor);
     }
 
     /// <summary>
     /// Writes the end constructor.
     /// </summary>
-    public void WriteEndConstructor()
+    public virtual void WriteEndConstructor()
     {
       AutoCompleteClose(JsonToken.EndConstructor);
     }
@@ -280,13 +372,14 @@ namespace Newtonsoft.Json
     /// <param name="name">The name of the property.</param>
     public virtual void WritePropertyName(string name)
     {
+      _currentPosition.PropertyName = name;
       AutoComplete(JsonToken.PropertyName);
     }
 
     /// <summary>
     /// Writes the end of the current Json object or array.
     /// </summary>
-    public void WriteEnd()
+    public virtual void WriteEnd()
     {
       WriteEnd(Peek());
     }
@@ -329,7 +422,7 @@ namespace Newtonsoft.Json
           case JsonToken.StartConstructor:
             string constructorName = reader.Value.ToString();
             // write a JValue date when the constructor is for a date
-            if (string.Compare(constructorName, "Date", StringComparison.Ordinal) == 0)
+            if (string.Equals(constructorName, "Date", StringComparison.Ordinal))
               WriteConstructorDate(reader);
             else
               WriteStartConstructor(reader.Value.ToString());
@@ -341,16 +434,16 @@ namespace Newtonsoft.Json
             WriteComment(reader.Value.ToString());
             break;
           case JsonToken.Integer:
-            WriteValue((long)reader.Value);
+            WriteValue(Convert.ToInt64(reader.Value, CultureInfo.InvariantCulture));
             break;
           case JsonToken.Float:
-            WriteValue((double)reader.Value);
+            WriteValue(Convert.ToDouble(reader.Value, CultureInfo.InvariantCulture));
             break;
           case JsonToken.String:
             WriteValue(reader.Value.ToString());
             break;
           case JsonToken.Boolean:
-            WriteValue((bool)reader.Value);
+            WriteValue(Convert.ToBoolean(reader.Value, CultureInfo.InvariantCulture));
             break;
           case JsonToken.Null:
             WriteNull();
@@ -389,17 +482,17 @@ namespace Newtonsoft.Json
     private void WriteConstructorDate(JsonReader reader)
     {
       if (!reader.Read())
-        throw new Exception("Unexpected end while reading date constructor.");
+        throw JsonWriterException.Create(this, "Unexpected end when reading date constructor.", null);
       if (reader.TokenType != JsonToken.Integer)
-        throw new Exception("Unexpected token while reading date constructor. Expected Integer, got " + reader.TokenType);
+        throw JsonWriterException.Create(this, "Unexpected token when reading date constructor. Expected Integer, got " + reader.TokenType, null);
 
       long ticks = (long)reader.Value;
       DateTime date = JsonConvert.ConvertJavaScriptTicksToDateTime(ticks);
 
       if (!reader.Read())
-        throw new Exception("Unexpected end while reading date constructor.");
+        throw JsonWriterException.Create(this, "Unexpected end when reading date constructor.", null);
       if (reader.TokenType != JsonToken.EndConstructor)
-        throw new Exception("Unexpected token while reading date constructor. Expected EndConstructor, got " + reader.TokenType);
+        throw JsonWriterException.Create(this, "Unexpected token when reading date constructor. Expected EndConstructor, got " + reader.TokenType, null);
 
       WriteValue(date);
     }
@@ -430,110 +523,124 @@ namespace Newtonsoft.Json
       }
     }
 
-    private void WriteEnd(JTokenType type)
+    private void WriteEnd(JsonContainerType type)
     {
       switch (type)
       {
-        case JTokenType.Object:
+        case JsonContainerType.Object:
           WriteEndObject();
           break;
-        case JTokenType.Array:
+        case JsonContainerType.Array:
           WriteEndArray();
           break;
-        case JTokenType.Constructor:
+        case JsonContainerType.Constructor:
           WriteEndConstructor();
           break;
         default:
-          throw new JsonWriterException("Unexpected type when writing end: " + type);
+          throw JsonWriterException.Create(this, "Unexpected type when writing end: " + type, null);
       }
     }
 
     private void AutoCompleteAll()
     {
-      while (_top > 0)
+      while (Top > 0)
       {
         WriteEnd();
       }
     }
 
-    private JTokenType GetTypeForCloseToken(JsonToken token)
+    private JsonContainerType GetTypeForCloseToken(JsonToken token)
     {
       switch (token)
       {
         case JsonToken.EndObject:
-          return JTokenType.Object;
+          return JsonContainerType.Object;
         case JsonToken.EndArray:
-          return JTokenType.Array;
+          return JsonContainerType.Array;
         case JsonToken.EndConstructor:
-          return JTokenType.Constructor;
+          return JsonContainerType.Constructor;
         default:
-          throw new JsonWriterException("No type for token: " + token);
+          throw JsonWriterException.Create(this, "No type for token: " + token, null);
       }
     }
 
-    private JsonToken GetCloseTokenForType(JTokenType type)
+    private JsonToken GetCloseTokenForType(JsonContainerType type)
     {
       switch (type)
       {
-        case JTokenType.Object:
+        case JsonContainerType.Object:
           return JsonToken.EndObject;
-        case JTokenType.Array:
+        case JsonContainerType.Array:
           return JsonToken.EndArray;
-        case JTokenType.Constructor:
+        case JsonContainerType.Constructor:
           return JsonToken.EndConstructor;
         default:
-          throw new JsonWriterException("No close token for type: " + type);
+          throw JsonWriterException.Create(this, "No close token for type: " + type, null);
       }
     }
 
     private void AutoCompleteClose(JsonToken tokenBeingClosed)
     {
       // write closing symbol and calculate new state
-
       int levelsToComplete = 0;
+      JsonContainerType type = GetTypeForCloseToken(tokenBeingClosed);
 
-      for (int i = 0; i < _top; i++)
+      if (_currentPosition.Type == type)
       {
-        int currentLevel = _top - i;
-
-        if (_stack[currentLevel] == GetTypeForCloseToken(tokenBeingClosed))
+        levelsToComplete = 1;
+      }
+      else
+      {
+        int top = Top - 2;
+        for (int i = top; i >= 0; i--)
         {
-          levelsToComplete = i + 1;
-          break;
+          int currentLevel = top - i;
+
+          if (_stack[currentLevel].Type == type)
+          {
+            levelsToComplete = i + 2;
+            break;
+          }
         }
       }
 
       if (levelsToComplete == 0)
-        throw new JsonWriterException("No token to close.");
+        throw JsonWriterException.Create(this, "No token to close.", null);
 
       for (int i = 0; i < levelsToComplete; i++)
       {
         JsonToken token = GetCloseTokenForType(Pop());
 
-        if (_currentState != State.ObjectStart && _currentState != State.ArrayStart)
-          WriteIndent();
+        if (_currentState == State.Property)
+          WriteNull();
+
+        if (_formatting == Formatting.Indented)
+        {
+          if (_currentState != State.ObjectStart && _currentState != State.ArrayStart)
+            WriteIndent();
+        }
 
         WriteEnd(token);
-      }
 
-      JTokenType currentLevelType = Peek();
+        JsonContainerType currentLevelType = Peek();
 
-      switch (currentLevelType)
-      {
-        case JTokenType.Object:
-          _currentState = State.Object;
-          break;
-        case JTokenType.Array:
-          _currentState = State.Array;
-          break;
-        case JTokenType.Constructor:
-          _currentState = State.Array;
-          break;
-        case JTokenType.None:
-          _currentState = State.Start;
-          break;
-        default:
-          throw new JsonWriterException("Unknown JsonType: " + currentLevelType);
+        switch (currentLevelType)
+        {
+          case JsonContainerType.Object:
+            _currentState = State.Object;
+            break;
+          case JsonContainerType.Array:
+            _currentState = State.Array;
+            break;
+          case JsonContainerType.Constructor:
+            _currentState = State.Array;
+            break;
+          case JsonContainerType.None:
+            _currentState = State.Start;
+            break;
+          default:
+            throw JsonWriterException.Create(this, "Unknown JsonType: " + currentLevelType, null);
+        }
       }
     }
 
@@ -568,31 +675,16 @@ namespace Newtonsoft.Json
 
     internal void AutoComplete(JsonToken tokenBeingWritten)
     {
-      int token;
-
-      switch (tokenBeingWritten)
-      {
-        default:
-          token = (int)tokenBeingWritten;
-          break;
-        case JsonToken.Integer:
-        case JsonToken.Float:
-        case JsonToken.String:
-        case JsonToken.Boolean:
-        case JsonToken.Null:
-        case JsonToken.Undefined:
-        case JsonToken.Date:
-        case JsonToken.Bytes:
-          // a value is being written
-          token = 7;
-          break;
-      }
+      if (tokenBeingWritten != JsonToken.StartObject
+        && tokenBeingWritten != JsonToken.StartArray
+        && tokenBeingWritten != JsonToken.StartConstructor)
+        UpdateScopeWithFinishedValue();
 
       // gets new state based on the current state and what is being written
-      State newState = stateArray[token][(int)_currentState];
+      State newState = StateArray[(int)tokenBeingWritten][(int)_currentState];
 
       if (newState == State.Error)
-        throw new JsonWriterException("Token {0} in state {1} would result in an invalid JavaScript object.".FormatWith(CultureInfo.InvariantCulture, tokenBeingWritten.ToString(), _currentState.ToString()));
+        throw JsonWriterException.Create(this, "Token {0} in state {1} would result in an invalid JSON object.".FormatWith(CultureInfo.InvariantCulture, tokenBeingWritten.ToString(), _currentState.ToString()), null);
 
       if ((_currentState == State.Object || _currentState == State.Array || _currentState == State.Constructor) && tokenBeingWritten != JsonToken.Comment)
       {
@@ -604,13 +696,16 @@ namespace Newtonsoft.Json
           WriteIndentSpace();
       }
 
-      WriteState writeState = WriteState;
-
-      // don't indent a property when it is the first token to be written (i.e. at the start)
-      if ((tokenBeingWritten == JsonToken.PropertyName && writeState != WriteState.Start) ||
-        writeState == WriteState.Array || writeState == WriteState.Constructor)
+      if (_formatting == Formatting.Indented)
       {
-        WriteIndent();
+        WriteState writeState = WriteState;
+
+        // don't indent a property when it is the first token to be written (i.e. at the start)
+        if ((tokenBeingWritten == JsonToken.PropertyName && writeState != WriteState.Start) ||
+            writeState == WriteState.Array || writeState == WriteState.Constructor)
+        {
+          WriteIndent();
+        }
       }
 
       _currentState = newState;
@@ -802,6 +897,24 @@ namespace Newtonsoft.Json
     }
 #endif
 
+    /// <summary>
+    /// Writes a <see cref="Guid"/> value.
+    /// </summary>
+    /// <param name="value">The <see cref="Guid"/> value to write.</param>
+    public virtual void WriteValue(Guid value)
+    {
+      AutoComplete(JsonToken.String);
+    }
+
+    /// <summary>
+    /// Writes a <see cref="TimeSpan"/> value.
+    /// </summary>
+    /// <param name="value">The <see cref="TimeSpan"/> value to write.</param>
+    public virtual void WriteValue(TimeSpan value)
+    {
+      AutoComplete(JsonToken.String);
+    }
+
     /// <summary>
     /// Writes a <see cref="Nullable{Int32}"/> value.
     /// </summary>
@@ -988,6 +1101,30 @@ namespace Newtonsoft.Json
     }
 #endif
 
+    /// <summary>
+    /// Writes a <see cref="Nullable{Guid}"/> value.
+    /// </summary>
+    /// <param name="value">The <see cref="Nullable{Guid}"/> value to write.</param>
+    public virtual void WriteValue(Guid? value)
+    {
+      if (value == null)
+        WriteNull();
+      else
+        WriteValue(value.Value);
+    }
+
+    /// <summary>
+    /// Writes a <see cref="Nullable{TimeSpan}"/> value.
+    /// </summary>
+    /// <param name="value">The <see cref="Nullable{TimeSpan}"/> value to write.</param>
+    public virtual void WriteValue(TimeSpan? value)
+    {
+      if (value == null)
+        WriteNull();
+      else
+        WriteValue(value.Value);
+    }
+
     /// <summary>
     /// Writes a <see cref="T:Byte[]"/> value.
     /// </summary>
@@ -1000,6 +1137,18 @@ namespace Newtonsoft.Json
         AutoComplete(JsonToken.Bytes);
     }
 
+    /// <summary>
+    /// Writes a <see cref="Uri"/> value.
+    /// </summary>
+    /// <param name="value">The <see cref="Uri"/> value to write.</param>
+    public virtual void WriteValue(Uri value)
+    {
+      if (value == null)
+        WriteNull();
+      else
+        AutoComplete(JsonToken.String);
+    }
+
     /// <summary>
     /// Writes a <see cref="Object"/> value.
     /// An error will raised if the value cannot be written as a single JSON token.
@@ -1012,9 +1161,9 @@ namespace Newtonsoft.Json
         WriteNull();
         return;
       }
-      else if (value is IConvertible)
+      else if (ConvertUtils.IsConvertible(value))
       {
-        IConvertible convertible = value as IConvertible;
+        IConvertible convertible = ConvertUtils.ToConvertible(value);
 
         switch (convertible.GetTypeCode())
         {
@@ -1063,9 +1212,11 @@ namespace Newtonsoft.Json
           case TypeCode.Decimal:
             WriteValue(convertible.ToDecimal(CultureInfo.InvariantCulture));
             return;
+#if !(NETFX_CORE || PORTABLE)
           case TypeCode.DBNull:
             WriteNull();
             return;
+#endif
         }
       }
 #if !PocketPC && !NET20
@@ -1080,8 +1231,23 @@ namespace Newtonsoft.Json
         WriteValue((byte[])value);
         return;
       }
+      else if (value is Guid)
+      {
+        WriteValue((Guid)value);
+        return;
+      }
+      else if (value is Uri)
+      {
+        WriteValue((Uri)value);
+        return;
+      }
+      else if (value is TimeSpan)
+      {
+        WriteValue((TimeSpan)value);
+        return;
+      }
 
-      throw new ArgumentException("Unsupported type: {0}. Use the JsonSerializer class to get the object's JSON representation.".FormatWith(CultureInfo.InvariantCulture, value.GetType()));
+      throw JsonWriterException.Create(this, "Unsupported type: {0}. Use the JsonSerializer class to get the object's JSON representation.".FormatWith(CultureInfo.InvariantCulture, value.GetType()), null);
     }
     #endregion
 
@@ -1103,7 +1269,7 @@ namespace Newtonsoft.Json
       if (ws != null)
       {
         if (!StringUtils.IsWhiteSpace(ws))
-          throw new JsonWriterException("Only white space characters should be used.");
+          throw JsonWriterException.Create(this, "Only white space characters should be used.", null);
       }
     }
 
@@ -1115,7 +1281,7 @@ namespace Newtonsoft.Json
 
     private void Dispose(bool disposing)
     {
-      if (WriteState != WriteState.Closed)
+      if (_currentState != State.Closed)
         Close();
     }
   }
