@@ -28,6 +28,7 @@
 
 using System;
 using System.Collections.Specialized;
+using System.Collections.Generic;
 
 namespace Meebey.SmartIrc4net
 {
@@ -759,4 +760,36 @@ namespace Meebey.SmartIrc4net
         }
     }
 
+    public class BounceEventArgs : IrcEventArgs
+    {
+        /// <summary>
+        /// Hostname/address of the server to which the user is being redirected.
+        /// May be null if not successfully parsed from the message.
+        /// </summary>
+        public string Server { get; private set; }
+
+        /// <summary>
+        /// Port of the server to which the user is being redirected.
+        /// May be -1 if not successfully parsed from the message.
+        /// </summary>
+        public int Port { get; private set; }
+
+        internal BounceEventArgs(IrcMessageData data, string server, int port) : base(data)
+        {
+            Server = server;
+            Port = Port;
+        }
+    }
+
+    public class ChannelModeChangeEventArgs : IrcEventArgs
+    {
+        public string Channel { get; private set; }
+        public List<ChannelModeChangeInfo> ModeChanges { get; private set; }
+
+        internal ChannelModeChangeEventArgs(IrcMessageData data, string channel, List<ChannelModeChangeInfo> modeChanges) : base(data)
+        {
+            Channel = channel;
+            ModeChanges = modeChanges;
+        }
+    }
 }
