@@ -42,7 +42,9 @@ namespace Twitterizer.Commands
     /// The Send Direct Message Command class
     /// </summary>
     [AuthorizedCommandAttribute]
+#if !SILVERLIGHT
     [Serializable]
+#endif
     internal sealed class SendDirectMessageCommand : TwitterCommand<TwitterDirectMessage>
     {
         /// <summary>
@@ -131,7 +133,7 @@ namespace Twitterizer.Commands
             this.RequestParameters.Add("text", this.Text);
 
             if (this.RecipientUserId > 0)
-                this.RequestParameters.Add("user_id", this.RecipientUserId.ToString(CultureInfo.InvariantCulture));
+                this.RequestParameters.Add("user_id", this.RecipientUserId.ToString("#"));
 
             if (!string.IsNullOrEmpty(this.RecipientUserName) && this.RecipientUserId <= 0)
                 this.RequestParameters.Add("screen_name", this.RecipientUserName);
