@@ -34,24 +34,14 @@
 
 namespace Twitterizer
 {
-    using System;
-
     /// <summary>
     /// The Status Update Options class
     /// </summary>
 #if !SILVERLIGHT
-    [Serializable]
+    [System.Serializable]
 #endif
     public sealed class StatusUpdateOptions : OptionalProperties
     {
-        /// <summary>
-        /// Initializes a new instance of the <see cref="StatusUpdateOptions"/> class.
-        /// </summary>
-        public StatusUpdateOptions()
-            : base()
-        {
-        }
-
         /// <summary>
         /// Gets or sets the in reply to status id.
         /// </summary>
@@ -81,5 +71,12 @@ namespace Twitterizer
         /// </summary>
         /// <value>The place id.</value>
         public string PlaceId { get; set; }
+
+        /// <summary>
+        /// Gets or sets a value indicating whether links should be wrapped with Twitter's t.co link wrapper.
+        /// </summary>
+        /// <value><c>true</c> if [wrap links]; otherwise, <c>false</c>.</value>
+        /// <remarks>More information is available here: https://dev.twitter.com/docs/tco-link-wrapper/faq</remarks>
+        public bool WrapLinks { get; set; }
     }
 }
