@@ -90,7 +90,7 @@ namespace Twitterizer.Commands
                 NumberFormatInfo nfi = CultureInfo.InvariantCulture.NumberFormat;
 
                 if (options.InReplyToStatusId > 0)
-                    this.RequestParameters.Add("in_reply_to_status_id", options.InReplyToStatusId.ToString(CultureInfo.CurrentCulture));
+                    this.RequestParameters.Add("in_reply_to_status_id", options.InReplyToStatusId.ToString("#"));
 
                 if (options.Latitude != 0)
                     this.RequestParameters.Add("lat", options.Latitude.ToString(nfi));
@@ -103,6 +103,9 @@ namespace Twitterizer.Commands
 
                 if (options.PlacePin)
                     this.RequestParameters.Add("display_coordinates", "true");
+
+                if (options.WrapLinks)
+                    this.RequestParameters.Add("wrap_links", "true");
             }
         }
     }
