@@ -1,32 +1,25 @@
-/*
- * $Id: FrontendManager.cs 378 2008-08-24 00:26:35Z meebey $
- * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Engine/FrontendManager.cs $
- * $Rev: 378 $
- * $Author: meebey $
- * $Date: 2008-08-24 02:26:35 +0200 (Sun, 24 Aug 2008) $
- *
- * Smuxi - Smart MUltipleXed Irc
- *
- * Copyright (c) 2008 Alan McGovern <alan.mcgovern@gmail.com>
- * Copyright (c) 2008 Mirco Bauer <meebey@meebey.net>
- *
- * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- */
-
+// This file is part of Smuxi and is licensed under the terms of MIT/X11
+//
+// Copyright (c) 2008 Alan McGovern <alan.mcgovern@gmail.com>
+// Copyright (c) 2008 Mirco Bauer <meebey@meebey.net>
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+// THE SOFTWARE.
 using System;
 using System.Collections.Generic;
 using System.Text;
@@ -255,7 +248,9 @@ namespace Smuxi.Common
                 thread.Abort();
                 InitThread();
             }
-            tasks.Clear();
+            lock (tasks) {
+                tasks.Clear();
+            }
         }
         
         public void Reset()
