@@ -121,7 +121,9 @@ namespace Smuxi.Engine
             Session.AddChat(NetworkChat);
             Session.SyncChat(NetworkChat);
             var msg = _("Connecting to campfire... ");
-            fm.SetStatus(msg);
+            if (fm != null) {
+                fm.SetStatus(msg);
+            }
             var bld = CreateMessageBuilder().AppendEventPrefix().AppendText(msg);
             Session.AddMessageToChat(NetworkChat, bld.ToMessage());
 
@@ -162,7 +164,9 @@ namespace Smuxi.Engine
 
             Client.Credentials = new NetworkCredential(Key, "X");
             msg = _("Connected to campfire");
-            fm.SetStatus(msg);
+            if (fm != null) {
+                fm.SetStatus(msg);
+            }
             bld = CreateMessageBuilder().AppendEventPrefix().AppendText(msg);
             Session.AddMessageToChat(NetworkChat, bld.ToMessage());
 
@@ -194,19 +198,21 @@ namespace Smuxi.Engine
             // TRANSLATOR: this line is used as a label / category for a
             // list of commands below
             builder.AppendHeader(_("Campfire Commands"));
-            cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
+            Session.AddMessageToFrontend(cd, builder.ToMessage());
 
             string[] help = {
                 "connect campfire username password",
-                "list",
+                "join",
+                "topic",
                 "uploads",
+                "say"
             };
 
             foreach (string line in help) {
                 builder = CreateMessageBuilder();
                 builder.AppendEventPrefix();
                 builder.AppendText(line);
-                cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
+                Session.AddMessageToFrontend(cd, builder.ToMessage());
             }
         }
 
@@ -353,7 +359,16 @@ namespace Smuxi.Engine
         {
             var message = new MessageSending { body = text, type = Campfire.MessageType.TextMessage};
             var wrapper = new MessageWrapper { message = message };
-            var res = Client.Post<MessageResponse>(String.Format("/room/{0}/speak.json", chat.ID), wrapper).Message;
+            Message res;
+            try {
+                res = Client.Post<MessageResponse>(String.Format("/room/{0}/speak.json", chat.ID), wrapper).Message;
+            } catch (WebServiceException e) {
+                var bld = CreateMessageBuilder();
+                bld.AppendErrorText(_("Failed to post message: {0}"), e.Message);
+                Session.AddMessageToChat(NetworkChat, bld.ToMessage());
+                return;
+            }
+
             ShowMessage(this, new MessageReceivedEventArgs(chat, res));
             LastSentId = res.Id;
         }
@@ -469,6 +484,15 @@ namespace Smuxi.Engine
             Session.AddMessageToChat(chat, bld.ToMessage());
     }
 
+        void ShowError(object sender, ErrorReceivedEventArgs args)
+        {
+            var message = args.StatusDescription;
+            var bld = CreateMessageBuilder();
+
+            bld.AppendErrorText(_("Error reading from stream: {0}"), message);
+            Session.AddMessageToChat(NetworkChat, bld.ToMessage());
+        }
+
         public override void OpenChat(FrontendManager fm, ChatModel chat_)
         {
             Trace.Call(fm, chat_);
@@ -508,6 +532,7 @@ namespace Smuxi.Engine
                 EventStreams.Add(chat, stream);
 
             stream.MessageReceived += ShowMessage;
+            stream.ErrorReceived += ShowError;
             stream.Start();
         }
 
