@@ -60,11 +60,11 @@ namespace Smuxi.Engine
             foreach (string server in servers) {
                 string[] serverParts = server.Split(new char[] {'/'});
                 string protocol = serverParts[0];
-                string servername = serverParts[1];
-                ServerModel ser = GetServer(protocol, servername);
+                string serverId = serverParts[1];
+                ServerModel ser = GetServer(protocol, serverId);
                 if (ser == null) {
 #if LOG4NET
-                    _Logger.Error("GetServerList(): GetServer(" + protocol + ", " + servername +") returned null! ignoring...");
+                    _Logger.Error("GetServerList(): GetServer(" + protocol + ", " + serverId +") returned null! ignoring...");
 #endif 
                     continue;
                 }
@@ -84,25 +84,12 @@ namespace Smuxi.Engine
                 throw new ArgumentNullException("servername");
             }
             
-            string prefix = "Servers/" + protocol + "/" + servername + "/";
             ServerModel server = new ServerModel();
-            if (_UserConfig[prefix + "Hostname"] == null) {
-                // server does not exist
+            try {
+                server.Load(_UserConfig, protocol, servername);
+            } catch (ArgumentException) {
                 return null;
             }
-            server.Protocol    = protocol;
-            server.Hostname    = (string) _UserConfig[prefix + "Hostname"];
-            server.Port        = (int)    _UserConfig[prefix + "Port"];
-            server.Network     = (string) _UserConfig[prefix + "Network"];
-            server.Username    = (string) _UserConfig[prefix + "Username"];
-            server.Password    = (string) _UserConfig[prefix + "Password"];
-            server.UseEncryption = (bool) _UserConfig[prefix + "UseEncryption"];
-            server.ValidateServerCertificate =
-                (bool) _UserConfig[prefix + "ValidateServerCertificate"];
-            if (_UserConfig[prefix + "OnStartupConnect"] != null) {
-                server.OnStartupConnect = (bool) _UserConfig[prefix + "OnStartupConnect"];
-            }
-            server.OnConnectCommands  = _UserConfig[prefix + "OnConnectCommands"] as IList<string>;
             return server;
         }
         
@@ -153,34 +140,31 @@ namespace Smuxi.Engine
             if (server.Hostname.Contains("\n")) {
                 throw new InvalidOperationException(_("Server hostname contains invalid characters (newline)."));
             }
+            var highestServerId = -1;
             foreach (var s in GetServerList()) {
                 if (s.Protocol == server.Protocol &&
-                    s.Hostname == server.Hostname) {
+                    s.ServerID == server.ServerID) {
                     throw new InvalidOperationException(
-                        String.Format(_("Server '{0}' already exists."),
-                                      server.Hostname)
+                        String.Format(_("Server ID '{0}' already exists."),
+                                      server.ServerID)
                     );
                 }
+                int id;
+                if (Int32.TryParse(s.ServerID, out id) && id > highestServerId) {
+                    highestServerId = id;
+                }
             }
-
-            string prefix = "Servers/" + server.Protocol + "/" + server.Hostname + "/";
-            _UserConfig[prefix + "Hostname"] = server.Hostname;
-            _UserConfig[prefix + "Port"]     = server.Port;
-            _UserConfig[prefix + "Network"]  = server.Network;
-            _UserConfig[prefix + "Username"] = server.Username;
-            _UserConfig[prefix + "Password"] = server.Password;
-            _UserConfig[prefix + "UseEncryption"] = server.UseEncryption;
-            _UserConfig[prefix + "ValidateServerCertificate"] =
-                server.ValidateServerCertificate;
-            _UserConfig[prefix + "OnStartupConnect"] = server.OnStartupConnect;
-            _UserConfig[prefix + "OnConnectCommands"] = server.OnConnectCommands;
+            if (String.IsNullOrEmpty(server.ServerID)) {
+                server.ServerID = (++highestServerId).ToString();
+            }
+            server.Save(_UserConfig);
             
             string[] servers = (string[]) _UserConfig["Servers/Servers"];
             if (servers == null) {
                 servers = new string[] {};
             }
             List<string> serverList = new List<string>(servers);
-            serverList.Add(server.Protocol + "/" + server.Hostname);
+            serverList.Add(server.Protocol + "/" + server.ServerID);
             _UserConfig["Servers/Servers"] = serverList.ToArray();
         }
 
@@ -191,18 +175,7 @@ namespace Smuxi.Engine
             if (server == null) {
                 throw new ArgumentNullException("server");
             }
-            
-            string prefix = "Servers/" + server.Protocol + "/" + server.Hostname + "/";
-            _UserConfig[prefix + "Hostname"] = server.Hostname;
-            _UserConfig[prefix + "Port"]     = server.Port;
-            _UserConfig[prefix + "Network"]  = server.Network;
-            _UserConfig[prefix + "Username"] = server.Username;
-            _UserConfig[prefix + "Password"] = server.Password;
-            _UserConfig[prefix + "UseEncryption"] = server.UseEncryption;
-            _UserConfig[prefix + "ValidateServerCertificate"] =
-                server.ValidateServerCertificate;
-            _UserConfig[prefix + "OnStartupConnect"] = server.OnStartupConnect;
-            _UserConfig[prefix + "OnConnectCommands"] = server.OnConnectCommands;
+            server.Save(_UserConfig);
         }
         
         public void RemoveServer(string protocol, string servername)
