@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006, 2010-2011, 2013 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2006, 2010-2011, 2013, 2015 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -36,8 +36,10 @@ namespace Smuxi.Engine
         private Config    _Config;
         private string    _UserPrefix;
         private string    _DefaultPrefix = "Engine/Users/DEFAULT/";
-        private Hashtable _Cache; 
-        
+        private Hashtable _Cache;
+
+        public FrontendConfig FrontendConfig { get; set; }
+
         public event EventHandler<ConfigChangedEventArgs> Changed;
 
         public bool IsCaching
@@ -53,21 +55,32 @@ namespace Smuxi.Engine
                 }
             }
         }
-        
+
+        /// <remarks>
+        /// This property is thread-safe.
+        /// </remarks>
         public object this[string key]
         {
             get {
+                object obj = null;
+                // allow engine config to be overriden by frontend config
+                if (FrontendConfig != null) {
+                    obj = FrontendConfig[key];
+                    if (obj != null) {
+                        return obj;
+                    }
+                }
+
                 if (IsCaching) {
                     if (_Cache.Contains(key)) {
                         return _Cache[key];
                     }
                 }
                 
-                object obj;
                 obj = _Config[_UserPrefix + key];
                 if (obj != null) {
                     if (IsCaching) {
-                        _Cache.Add(key, obj);
+                        _Cache[key] = obj;
                     }
                     return obj;
                 }
@@ -79,12 +92,17 @@ namespace Smuxi.Engine
                 }
 #endif
                 if (IsCaching) {
-                    _Cache.Add(key, obj);
+                    _Cache[key] = obj;
                 }
 
                 return obj;
             }
             set {
+                // ignore writing back overridden config keys
+                if (FrontendConfig != null && FrontendConfig[key] != null) {
+                    return;
+                }
+
                 // TODO: make remoting calls after a timeout and batch the update
                 _Config[_UserPrefix + key] = value;
                 
@@ -167,7 +185,7 @@ namespace Smuxi.Engine
 
             var start = DateTime.UtcNow;
             var conf = _Config.GetAll();
-            var cache = new Hashtable(conf.Count);
+            var cache = Hashtable.Synchronized(new Hashtable(conf.Count));
             foreach (var entry in conf) {
                 if (!entry.Key.StartsWith(_UserPrefix)) {
                     // no need to cache values of other users
