@@ -42,8 +42,8 @@ namespace Smuxi.Frontend.Gnome
                 Version = String.Format("\n Vendor: {0}", distVersion);
             }
             Version += "\n Frontend: " + Frontend.UIName + " " + version  +
-                       "\n Engine: " + Frontend.EngineVersion;
-            Copyright = "Copyright © 2005-2013 Mirco Bauer <meebey@meebey.net>";
+                       "\n Engine: " + Frontend.EngineAssemblyVersion;
+            Copyright = "Copyright © 2005-2015 Mirco Bauer <meebey@meebey.net> and other contributors";
             Authors = new string[] {
                 "Mirco Bauer <meebey@meebey.net>",
                 "David Paleino <dapal@debian.org>",
@@ -67,9 +67,15 @@ namespace Smuxi.Frontend.Gnome
             Logo = Frontend.LoadIcon(
                 Frontend.IconName, 256, "icon_256x256.png"
             );
-            // HACK: shows "not implemented" error on OS X
-            if (!Frontend.IsMacOSX) {
-                Website = "http://www.smuxi.org/";
+            // HACK: shows "not implemented" error on OS X and
+            // "No application is registered as handling this file" on Windows.
+            // This probably relies on gvfs or similar which isn't available in
+            // the GTK{+,#} ports/installers for OS X and Windows. Thus we only
+            // show the website URL as label instead.
+            if (Frontend.IsMacOSX || Frontend.IsWindows) {
+                WebsiteLabel = "https://smuxi.im/";
+            } else {
+                Website = "https://smuxi.im/";
                 WebsiteLabel = _("Smuxi Website");
             }
         }
