@@ -1,6 +1,6 @@
 // Smuxi - Smart MUltipleXed Irc
 // 
-// Copyright (c) 2010-2013 Mirco Bauer <meebey@meebey.net>
+// Copyright (c) 2010-2014 Mirco Bauer <meebey@meebey.net>
 // 
 // Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
 // 
@@ -31,6 +31,7 @@ namespace Smuxi.Frontend.Gnome
         Gtk.ListStore f_NetworkListStore;
 
         string ServerID { get; set; }
+        ServerModel Server { get; set; }
 
         public Gtk.Entry HostnameEntry {
             get {
@@ -72,6 +73,18 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        public Gtk.Entry NicknameEntry {
+            get {
+                return f_NicknameEntry;
+            }
+        }
+
+        public Gtk.Entry RealnameEntry {
+            get {
+                return f_RealnameEntry;
+            }
+        }
+
         public Gtk.CheckButton OnStartupConnectCheckButton {
             get {
                 return f_OnStartupConnectCheckButton;
@@ -94,7 +107,29 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
-        public bool ShowPassword {
+        public bool ShowNickname {
+            set {
+                // Smuxi < 0.11 does not support server specific nickname
+                if (Frontend.EngineProtocolVersion < new Version(0, 11)) {
+                    value = false;
+                }
+                f_NicknameLabel.Visible = value;
+                f_NicknameEntry.Visible = value;
+            }
+        }
+
+        public bool ShowRealname {
+            set {
+                // Smuxi < 0.11 does not support server specific realname
+                if (Frontend.EngineProtocolVersion < new Version(0, 11)) {
+                    value = false;
+                }
+                f_RealnameLabel.Visible = value;
+                f_RealnameEntry.Visible = value;
+            }
+        }
+
+       public bool ShowPassword {
             set {
                 f_PasswordLabel.Visible = value;
                 f_PasswordEntry.Visible = value;
@@ -128,6 +163,8 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call(server);
 
+            Server = server;
+
             // protocol is part of the PKEY, not allowed to change
             f_ProtocolComboBox.Sensitive = false;
 
@@ -135,10 +172,24 @@ namespace Smuxi.Frontend.Gnome
             ServerID = server.ServerID;
             f_HostnameEntry.Text = server.Hostname;
             f_NetworkComboBoxEntry.Entry.Text = server.Network;
+            if (String.IsNullOrEmpty(server.Nickname)) {
+                var defaultNicknames = (string[]) Frontend.UserConfig["Connection/Nicknames"];
+                f_NicknameEntry.Text = String.Join(" ", defaultNicknames);
+            } else {
+                f_NicknameEntry.Text = server.Nickname;
+            }
+            if (String.IsNullOrEmpty(server.Realname)) {
+                var defaultRealname = (string) Frontend.UserConfig["Connection/Realname"];
+                f_RealnameEntry.Text = defaultRealname;
+            } else {
+                f_RealnameEntry.Text = server.Realname;
+            }
             f_UsernameEntry.Text = server.Username;
             // HACK: Twitter username is part of the PKEY, not allowed to change
             if (server.Protocol == "Twitter") {
                 f_UsernameEntry.Sensitive = false;
+            } else {
+                f_UsernameEntry.Sensitive = true;
             }
             f_PasswordEntry.Text = server.Password;
             f_UseEncryptionCheckButton.Active = server.UseEncryption;
@@ -161,7 +212,10 @@ namespace Smuxi.Frontend.Gnome
         
         public ServerModel GetServer()
         {
-            ServerModel server = new ServerModel();
+            var server = Server;
+            if (server == null) {
+                server = new ServerModel();
+            }
             server.Protocol = f_ProtocolComboBox.ActiveText;
             server.ServerID = ServerID;
             server.Hostname = f_HostnameEntry.Text.Trim();
@@ -173,6 +227,8 @@ namespace Smuxi.Frontend.Gnome
                 server.Hostname = server.Username;
             }
             server.Password = f_PasswordEntry.Text;
+            server.Nickname = f_NicknameEntry.Text.Trim();
+            server.Realname = f_RealnameEntry.Text.Trim();
             server.UseEncryption = f_UseEncryptionCheckButton.Active;
             server.ValidateServerCertificate =
                 f_ValidateServerCertificateCheckButton.Active;
@@ -180,6 +236,8 @@ namespace Smuxi.Frontend.Gnome
             if (f_OnConnectCommandsTextView.Sensitive) {
                 server.OnConnectCommands =
                     f_OnConnectCommandsTextView.Buffer.Text.Split('\n');
+            } else {
+                server.OnConnectCommands = new List<string>();
             }
 
             return server;
@@ -247,6 +305,11 @@ namespace Smuxi.Frontend.Gnome
             f_UseEncryptionCheckButton.Clicked += delegate {
                 CheckUseEncryptionCheckButton();
             };
+
+            var defaultNicknames = (string[]) Frontend.UserConfig["Connection/Nicknames"];
+            f_NicknameEntry.Text = String.Join(" ", defaultNicknames);
+            var defaultRealname = (string) Frontend.UserConfig["Connection/Realname"];
+            f_RealnameEntry.Text = defaultRealname;
         }
 
         protected virtual void CheckIgnoreOnConnectCommandsCheckButton()
@@ -301,6 +364,8 @@ namespace Smuxi.Frontend.Gnome
                 case "IRC":
                     ShowHostname = true;
                     ShowNetwork = true;
+                    ShowNickname = true;
+                    ShowRealname = true;
                     ShowPassword = true;
                     SupportUseEncryption = true;
 
@@ -310,9 +375,21 @@ namespace Smuxi.Frontend.Gnome
                     f_PortSpinButton.Value = 6667;
                     f_PortSpinButton.Sensitive = true;
                     break;
+                case "Facebook":
+                    ShowHostname = false;
+                    ShowNetwork = false;
+                    ShowNickname = false;
+                    ShowRealname = false;
+                    ShowPassword = true;
+                    SupportUseEncryption = true;
+                    f_HostnameEntry.Text = "chat.facebook.com";
+                    f_PortSpinButton.Value = 5222;
+                    break;
                 case "XMPP":
                     ShowHostname = true;
                     ShowNetwork = false;
+                    ShowNickname = false;
+                    ShowRealname = false;
                     ShowPassword = true;
                     SupportUseEncryption = true;
                 
@@ -329,6 +406,8 @@ namespace Smuxi.Frontend.Gnome
                 case "MSNP":
                     ShowHostname = false;
                     ShowNetwork = false;
+                    ShowNickname = false;
+                    ShowRealname = false;
                     ShowPassword = true;
                     SupportUseEncryption = false;
 
@@ -343,6 +422,8 @@ namespace Smuxi.Frontend.Gnome
                 case "Twitter":
                     ShowHostname = false;
                     ShowNetwork = false;
+                    ShowNickname = false;
+                    ShowRealname = false;
                     ShowPassword = false;
                     SupportUseEncryption = true;
                     // engine always uses https
@@ -358,6 +439,8 @@ namespace Smuxi.Frontend.Gnome
                 case "Campfire":
                     ShowHostname = true;
                     ShowNetwork = false;
+                    ShowNickname = false;
+                    ShowRealname = false;
                     ShowPassword = true;
                     SupportUseEncryption = true;
                     // engine always uses https
@@ -374,6 +457,8 @@ namespace Smuxi.Frontend.Gnome
                 case "JabbR":
                     ShowHostname = true;
                     ShowNetwork = false;
+                    ShowNickname = false;
+                    ShowRealname = false;
                     ShowPassword = true;
                     SupportUseEncryption = true;
 
@@ -390,6 +475,8 @@ namespace Smuxi.Frontend.Gnome
                 default:
                     ShowHostname = true;
                     ShowNetwork = true;
+                    ShowNickname = true;
+                    ShowRealname = true;
                     ShowPassword = true;
                     SupportUseEncryption = true;
 
