@@ -237,6 +237,10 @@ namespace Smuxi.Frontend.Stfl
                         GC.Collect();
                         handled = true;
                         break;
+                    case "generate_messages":
+                        CommandGenerateMessages(cmd);
+                        handled = true;
+                        break;
                 }
             }
             return handled;
@@ -252,7 +256,6 @@ namespace Smuxi.Frontend.Stfl
             chatView.AddMessage(builder.ToMessage());
 
             string[] help = {
-                "help",
                 "window (number|close)",
                 "exit",
             };
@@ -292,6 +295,12 @@ namespace Smuxi.Frontend.Stfl
             cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg);
         }
 
+        void CommandGenerateMessages(CommandModel cmd)
+        {
+            var chat = f_ChatViewManager.CurrentChat;
+            CommandManager.CommandGenerateMessages(cmd, chat);
+        }
+
         // gets the position of the first space left
         private int GetLeftSpace(int end)
         {
