@@ -1,6 +1,6 @@
 // Smuxi - Smart MUltipleXed Irc
 // 
-// Copyright (c) 2010, 2012-2013 Mirco Bauer <meebey@meebey.net>
+// Copyright (c) 2010, 2012-2014 Mirco Bauer <meebey@meebey.net>
 // 
 // Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
 // 
@@ -19,6 +19,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 using System;
+using System.IO;
 using System.Linq;
 using System.Reflection;
 using System.Collections.Generic;
@@ -52,6 +53,8 @@ namespace Smuxi.Frontend
         TaskQueue       f_TaskQueue;
         TimeSpan        f_LastCommandTimeSpan;
 
+        public Version EngineProtocolVersion { get; set; }
+
         public TimeSpan LastCommandTimeSpan {
             get {
                 return f_LastCommandTimeSpan;
@@ -113,7 +116,18 @@ namespace Smuxi.Frontend
             }
 
             f_TaskQueue.Queue(delegate {
-                DoExecute(cmd);
+                try {
+                    DoExecute(cmd);
+                } catch (Exception ex) {
+#if LOG4NET
+                    f_Logger.Error("Execute(): DoExecute() threw exception!", ex);
+#endif
+                    var msg = CreateMessageBuilder().
+                        AppendErrorText("Command '{0}' failed. Reason: {1}",
+                                        cmd.Command, ex.Message).
+                        ToMessage();
+                    AddMessageToFrontend(cmd, msg);
+                }
             });
         }
 
@@ -132,6 +146,12 @@ namespace Smuxi.Frontend
                         CommandEcho(cmd);
                         handled = true;
                         break;
+                    case "benchmark_message_builder":
+                        CommandBenchmarkMessageBuilder(cmd);
+                        handled = true;
+                        break;
+                    case "exception":
+                        throw new Exception("You asked for it.");
                 }
             }
             if (handled) {
@@ -145,7 +165,7 @@ namespace Smuxi.Frontend
             handled = f_Session.Command(cmd);
             IProtocolManager pm = null;
             if (!handled) {
-                if (cmd.Chat is SessionChatModel) {
+                if (cmd.Chat is SessionChatModel && cmd.FrontendManager != null) {
                     pm = cmd.FrontendManager.CurrentProtocolManager;
                 } else {
                     pm = cmd.Chat.ProtocolManager;
@@ -163,6 +183,7 @@ namespace Smuxi.Frontend
                 var hooks = new HookRunner("frontend", "command-manager",
                                            "command-" + filteredCmd);
                 hooks.EnvironmentVariables.Add("FRONTEND_VERSION", FrontendVersion);
+                hooks.Environments.Add(new CommandHookEnvironment(cmd));
                 hooks.Environments.Add(new ChatHookEnvironment(cmd.Chat));
                 if (pm != null) {
                     hooks.Environments.Add(new ProtocolManagerHookEnvironment(pm));
@@ -230,12 +251,12 @@ namespace Smuxi.Frontend
                         output = String.Format("{0}{1}",
                                                cmd.CommandCharacter, output);
                     }
-                    Execute(new CommandModel(cmd.FrontendManager,
-                                             cmd.Chat,
-                                             cmd.CommandCharacter, output));
+                    DoExecute(new CommandModel(cmd.FrontendManager,
+                                               cmd.Chat,
+                                               cmd.CommandCharacter, output));
                 } else {
-                    var msg = new MessageBuilder().AppendText(output).ToMessage();
-                    f_Session.AddMessageToFrontend(cmd, msg);
+                    var msg = CreateMessageBuilder().AppendText(output).ToMessage();
+                    AddMessageToFrontend(cmd, msg);
                 }
             };
 
@@ -243,8 +264,8 @@ namespace Smuxi.Frontend
             string args = null;
             if (Environment.OSVersion.Platform == PlatformID.Unix) {
                 file = "sh";
-                args = String.Format("-c '{0}'",
-                                     parameter.Replace("'", @"\'"));
+                args = String.Format("-c \"{0}\"",
+                                     parameter.Replace("\"", @"\"""));
             } else {
                 file = parameters[1];
                 if (parameters.Length > 1) {
@@ -278,11 +299,11 @@ namespace Smuxi.Frontend
                     if (!String.IsNullOrEmpty(info.Arguments)) {
                         command += " " + info.Arguments;
                     }
-                    var msg = new MessageBuilder().
+                    var msg = CreateMessageBuilder().
                         AppendErrorText("Executing '{0}' failed with: {1}",
                                         command, ex.Message).
                         ToMessage();
-                    f_Session.AddMessageToFrontend(cmd, msg);
+                    AddMessageToFrontend(cmd, msg);
                 }
             }
         }
@@ -291,11 +312,11 @@ namespace Smuxi.Frontend
         {
             Trace.Call(cmd);
 
-            var msg = new MessageBuilder().
+            var msg = CreateMessageBuilder().
                 AppendEventPrefix().
                     AppendText(cmd.Parameter).
                     ToMessage();
-            f_Session.AddMessageToFrontend(cmd, msg);
+            AddMessageToFrontend(cmd, msg);
         }
 
         public void CommandGenerateMessages(CommandModel cmd, IChatView chat)
@@ -305,7 +326,7 @@ namespace Smuxi.Frontend
             var count = 0;
             Int32.TryParse(cmd.Parameter, out count);
 
-            var builder = new MessageBuilder();
+            var builder = CreateMessageBuilder();
             var sender = new ContactModel("msg-tester", "msg-tester", "test", "test");
             builder.AppendMessage(sender, "time for a messsage generator command so I can test speed and memory usage");
             var text = builder.CreateText(" *formatted text* ");
@@ -326,7 +347,7 @@ namespace Smuxi.Frontend
             }
             stop = DateTime.UtcNow;
 
-            builder = new MessageBuilder();
+            builder = CreateMessageBuilder();
             builder.AppendText(
                 "IChatView.AddMessage(): count: {0} took: {1:0} ms avg: {2:0.00} ms",
                 count,
@@ -336,13 +357,115 @@ namespace Smuxi.Frontend
             chat.AddMessage(builder.ToMessage());
         }
 
+        public void CommandBenchmarkMessageBuilder(CommandModel cmd)
+        {
+            Trace.Call(cmd);
+
+            var count = 1000;
+            var showHelp = false;
+            var appendMessage = false;
+            var appendText = false;
+            var appendEvent = false;
+            var appendFormat = false;
+            var toMessage = false;
+            try {
+                var opts = new NDesk.Options.OptionSet() {
+                    { "c|count=", v => count = Int32.Parse(v) },
+                    { "m|append-message", v => appendMessage = true },
+                    { "t|append-text", v => appendText = true },
+                    { "e|append-event", v => appendEvent = true },
+                    { "f|append-format", v => appendFormat = true },
+                    { "T|to-message", v => toMessage = true },
+                };
+                opts.Add("h|?|help", x => {
+                    showHelp = true;
+                    var writer = new StringWriter();
+                    opts.WriteOptionDescriptions(writer);
+                    AddMessageToFrontend(
+                        cmd,
+                        CreateMessageBuilder().
+                            AppendHeader("{0} usage", cmd.Command).
+                            AppendText("\n").
+                            AppendText("Parameters:\n").
+                            AppendText(writer.ToString()).
+                            ToMessage()
+                    );
+                    return;
+                });
+                opts.Parse(cmd.Parameter.Split(' '));
+                if (showHelp) {
+                    return;
+                }
+            } catch (Exception ex) {
+                AddMessageToFrontend(
+                    cmd,
+                    CreateMessageBuilder().
+                        AppendErrorText("Invalid parameter: {0}", ex.Message).
+                        ToMessage()
+                );
+                return;
+            }
+
+            DateTime start, stop;
+            start = DateTime.UtcNow;
+            MessageBuilder builder;
+            for (var i = 0; i < count; i++) {
+                builder = CreateMessageBuilder();
+                if (appendMessage) {
+                    builder.AppendMessage("This is message with a link to https://www.smuxi.org/.");
+                }
+                if (appendText) {
+                    builder.AppendText("This is message with just text.");
+                }
+                if (appendEvent) {
+                    builder.AppendEventPrefix();
+                }
+                if (appendFormat) {
+                    builder.AppendFormat("{0} [{1}] has joined {2}",
+                                         "meebey3",
+                                         "~smuxi@31-18-115-252-dynip.superkabel.de",
+                                         "#smuxi-devel");
+                }
+                if (toMessage) {
+                    var msg = builder.ToMessage();
+                }
+            }
+            stop = DateTime.UtcNow;
+
+            builder = CreateMessageBuilder();
+            builder.AppendText("MessageBuilder().");
+            if (appendMessage) {
+                builder.AppendText("AppendMessage().");
+            }
+            if (appendText) {
+                builder.AppendText("AppendText().");
+            }
+            if (appendEvent) {
+                builder.AppendText("AppendEventPrefix().");
+            }
+            if (appendFormat) {
+                builder.AppendText("AppendFormat().");
+            }
+            if (toMessage) {
+                builder.AppendText("ToMessage()");
+            }
+            builder.AppendText(
+                " count: {1} took: {2:0} ms avg: {3:0.00} ms",
+                cmd.Data,
+                count,
+                (stop - start).TotalMilliseconds,
+                (stop - start).TotalMilliseconds / count
+            );
+            AddMessageToFrontend(cmd, builder.ToMessage());
+        }
+
         private void Unknown(CommandModel cmd)
         {
             var msg = CreateMessageBuilder().
                 AppendEventPrefix().
                 AppendText(_("Unknown Command: {0}"), cmd.Command).
                 ToMessage();
-            f_Session.AddMessageToFrontend(cmd, msg);
+            AddMessageToFrontend(cmd, msg);
         }
 
         void NotEnoughParameters(CommandModel cmd)
@@ -351,7 +474,7 @@ namespace Smuxi.Frontend
                 AppendEventPrefix().
                 AppendText(_("Not enough parameters for {0} command"), cmd.Command).
                 ToMessage();
-            f_Session.AddMessageToFrontend(cmd, msg);
+            AddMessageToFrontend(cmd, msg);
         }
 
         MessageBuilder CreateMessageBuilder()
@@ -359,6 +482,22 @@ namespace Smuxi.Frontend
             return new MessageBuilder();
         }
 
+        void AddMessageToFrontend(CommandModel cmd, MessageModel msg)
+        {
+            if (cmd == null) {
+                throw new ArgumentNullException("cmd");
+            }
+            if (msg == null) {
+                throw new ArgumentNullException("msg");
+            }
+
+            if (EngineProtocolVersion != null && EngineProtocolVersion >= new Version(0, 10)) {
+                f_Session.AddMessageToFrontend(cmd, msg);
+            } else {
+                f_Session.AddMessageToChat(cmd.Chat, msg);
+            }
+        }
+
         protected virtual void OnTaskQueueExceptionEvent(object sender, TaskQueueExceptionEventArgs e)
         {
             Trace.Call(sender, e);
